* {
  margin: 0;
}
html, body {
  height: 100%;
}
body {
	background-color: #fff;
}

#top_logo_div{
	position: absolute;
	z-index:1000;
	left: 20px;
	top: 10px;
	background: url(../img/catboxLogo170x52.png) no-repeat;
	width: 170px;
	height: 52px;
}

@media (-webkit-min-device-pixel-ratio: 2), (min-resolution: 192dpi) {    
    #top_logo_div {
    	background: url(../img/catboxLogo170x52@2.png) no-repeat;
    	background-size: 170px 52px;
    }
}

#master_container {
	/*margin-top: 130px;*/
	padding-top: 110px;
}

h2 {
	font-size: 25px;
	color: #aaa;
}

h3 {
	font-size: 23px;
	color: #274485;
}

.codebox_help_image {
	float: right;	max-width: 30%;
	height: auto;
}

.playback_help_image {
	margin-bottom: 20px;
	max-width: 100%;
	height: auto;
}

.top_control_bar_img {
	margin-bottom: 20px;
	max-width: 100%;
	height: auto;
}

.graph_help_image {
	margin-bottom: 20px;
	max-width: 100%;
	height: auto;
}

.invisible {
	display: none;
}

.visible {
	display: block;
}

.page-wrap {
  min-height: 100%;
  /* equal to footer height */
  margin-bottom: -80px; 
}
.page-wrap:after {
  content: "";
  display: block;
}
.fixed_bottom_nav, .page-wrap:after {
  height: 50px; 
}
.fixed_bottom_nav {
	position: fixed;
	bottom: 0px;
	width: 100%;
	height: 50px;
	background: #606060;
	border-top: 2px solid #333;
	line-height: 50px;
  /*background: -webkit-linear-gradient(left, #d7d7d7 0px, #bcbcbc 60%, #fff 100%);*/ /* For Safari 5.1 to 6.0 */
  /*background: -o-linear-gradient(right, #d7d7d7 0px, #bcbcbc 60%, #fff 100%);*/ /* For Opera 11.1 to 12.0 */
  /*background: -moz-linear-gradient(right, #d7d7d7 0px, #bcbcbc 60%, #fff 100%);*/ /* For Firefox 3.6 to 15 */
  /*background: linear-gradient(to right, #d7d7d7 0px, #bcbcbc 60%, #fff 100%);*/ /* Standard syntax */
}

.nav_item {
	text-align: center; 
	float: left; 
	height: 100%; 
	font-size: 20px; 
	padding-top: 13px; 
	padding-left: 15px; 
	padding-right: 15px; 
	color: #ececec;
	line-height: 50px;
}


.arrow_nav_item {
	background-color: #606060;
	text-align: center; 
	float: left; 
	height: 100%; 
	padding-top: 10px;
	padding-left: 30px; 
	padding-right: 15px;
	line-height: 50px;
}

.back_nav_item {
	background-color: #a0a0a0; 
	padding-right: 100px;
}

.nav_link {
	color: #ececec;
	cursor: pointer;
	line-height: 50px;
}

.arrow-left {
	width: 0; 
	height: 0; 
	border-top: 16px solid transparent;
	border-bottom: 16px solid transparent; 
	border-right: 22px solid red; 
	border-radius: 5px;
}

.small-arrow-left {
	width: 0;
	height: 0;
	border-top: 11px solid transparent;
	border-bottom: 11px solid transparent;
	border-right: 16px solid #d7d7d7;
	border-radius: 3px;
	position: relative;
	top: -27px;
	left: 4px;
}

.indent {
	padding-left: 30px;
}