* {
  margin: 0;
}
html, body {
  height: 100%;
}


body {
	background-color: #fff;
	color: #041133;
}

#master_container {
	margin-top: 20px;    /* Push down to accomodate logo */
}

.graph_div {
	text-align: center;
	padding-left: 0px;
	padding-right: 0px;
	
}

.graph_div a {
	color: #274485;
}


.clear3 {
	display: none;
	clear: both;
}

.clear4 {
	display: none;
	clear: both;
}


@media (max-width: 992px) {
  /* Represents portrait orientation styles */
  .clear3 {
    display: block;
    clear: both;
  }
  .chapter_row {
	padding-left: 40px;
  }
  .algo_title_row {
  	padding-left: 48px;
  }

  .graph_row {
    padding-left: 94px;
  }
}

@media (min-width: 992px) {
	/* Represents landscape orientation styles */
	.clear4 {
		display: block;
		clear: both;
	}
	.chapter_row, .algo_title_row {
	    padding-left: 20px;
    }

    .algo_title_row {
    	padding-left: 28px;
    }

	.graph_row {
	    padding-left: 76px;
	}
}





.indent {
	padding-left: 0px;
}

h2 {
	font-size: 32px;
	color: #979797;
}

h3 {
	font-size: 23px;
	color: #274485;
}

a:hover, a:focus {
	color: #274485;	/* Change this */
	text-decoration: none;
}

.graph_row {
	margin-top: 10px;
}

#top_logo_div{
	position: absolute;
	z-index:1000;
	left: 20px;
	background: url(../img/catboxLogo170x52.png) no-repeat;
	width: 170px;
	height: 52px;
}

@media (-webkit-min-device-pixel-ratio: 2), (min-resolution: 192dpi) {    
    #top_logo_div {
    	background: url(../img/catboxLogo170x52@2.png) no-repeat;
    	background-size: 170px 52px;
    }
}

.algo_div {
	background-color: white;
}

.graph_img {
	border: 1px solid #aaa;
	border-radius: 2px;
	height: 100%;
	width: auto;
}

.graph_div {
	text-align: left;
	border-radius: 0px;
}

.chapter_div {
	padding-top: 0px;
	border-radius: 0px;
	margin-top: 80px;
}

.invisible {
	display: none;
}

.visible {
	display: block;
}


.page-wrap {
  min-height: 100%;
  /* equal to footer height */
  margin-bottom: -80px; 
}
.page-wrap:after {
  content: "";
  display: block;
}
.fixed_bottom_nav, .page-wrap:after {
  height: 50px; 
}
.fixed_bottom_nav {
	position: fixed;
	bottom: 0px;
	width: 100%;
  /*background: -webkit-linear-gradient(left, #ded4b5 0px, #ded4b5 60%, #fff 100%); 
  background: -o-linear-gradient(right, #ded4b5 0px, #ded4b5 60%, #fff 100%);
  background: -moz-linear-gradient(right, #ded4b5 0px, #ded4b5 60%, #fff 100%); 
  background: linear-gradient(to right, #ded4b5 0px, #ded4b5 60%, #fff 100%); */
  	/*background-color: #333;*/
  	background-color: #606060;
  	border-top: 2px solid #333;
}

.nav_item {
	text-align: center; 
	float: left; 
	height: 100%; 
	font-size: 20px; 
	padding-top: 9px; 
	padding-left: 15px; 
	padding-right: 15px; 
	/*color: #e0e0e0;*/
	color: #ececec;
	/*background-color: #333;*/
	background-color: #606060;
	/*background-color: #274384;*/
}

.active {
	color: #041133;
	background-color: #ececec;
	/*color: #ececec;*/
	/*background-color: #333;*/
	/*background-color: #ded4b5;*/
}

.drop_up_holder {
	width:0px; 
	padding: 0px;
	float: left;
}

.nav_link {
	color: #e0e0e0;
	cursor: pointer;
}

.nav_drop_up {
	position: absolute;
	margin-left: 0px;
	padding-right: 10px;
	bottom: 50px;
	font-size: 18px;
	padding-left: 0px;
	/*background-color: #ded4b5;*/
	/*background-color: #333;*/
	background-color: #ececec;
	color: #041133;
	/*color: #ececec;*/
	border-top-right-radius: 4px;
	border-top-left-radius: 4px;
	/*border: 1px solid #333;*/
}

.nav_ul {
	list-style-type: none;
	padding-left: 10px;
}

.nav_ul li {
	text-align: left;
	padding-top: 5px;
	padding-bottom: 5px;
	border-bottom: 1px solid #999;
}

#alpha_nav {
	height: 514px;
}

.nav_ul a {
	color: #041133;
	/*color: #cdcdcd;*/
	text-decoration: none
}

.invisible {
	display: none;
}

.visible {
	display: block;
}

p {
	margin: 0 0 30px;
}